from __future__ import absolute_import, unicode_literals

from django.contrib.auth import authenticate

from ...auth.basic import (
    HttpResponseBasicChallenge,
    http_basic_credentials
)


class HTTPBasicMiddleWare(object):
    """This middleware checks the incoming request for a valid user/password
    combination. The request is rejected with 403 Forbidden if a valid user
    cannot be authenticated against the User db
    """
    def process_request(self, request):
        try:
            username, password = http_basic_credentials(request)
        except AttributeError:
            return HttpResponseBasicChallenge()

        user = authenticate(username=username, password=password)
        if user is None:
            return HttpResponseBasicChallenge()
        try:
            request.user.vidyoadmin.decrypt(password)
        except ValueError:
            raise Exception('user / vidyoadmin decryption mismatch')



from __future__ import absolute_import, unicode_literals

from django.contrib.admin import ModelAdmin, TabularInline
from django.core.exceptions import ValidationError, PermissionDenied, SuspiciousOperation
from django.core.validators import URLValidator, validate_ipv46_address, RegexValidator

from django import forms
from django.utils.translation import ugettext as _
from django.utils.encoding import python_2_unicode_compatible

from ..models import VidyoAdmin, IPModel


HostnameValidator = RegexValidator(URLValidator.host_re, message=_("Enter a valid hostname"))

#ASSUMPTIONS
#FIXME
# This code assumes that the VidyoAdmin object being edited is the
# same as request.user._dj_auth_user.


@python_2_unicode_compatible
class IPModelInline(TabularInline):
    """Whitelist IP address"""
    verbose_name_plural = _("Whitelisted hosts")
    verbose_name = _("IP address")
    model = IPModel
    extra = 1
    fk_name = '_admin_model'

    def clean(self, *args, **kwargs):
        self.model.clean(*args, **kwargs)

    def __str__(self):
        return _("Whitelist IP address")


class VidyoAdminForm(forms.ModelForm):
    """
    Custom form for the VidyoAdmin model with additional fields
    to allow the user to change their password.
    """
    class Meta:
        model = VidyoAdmin
        exclude = ()

    # Additional form fields which are on this form but not on the model
    current_admin_password = forms.CharField(
        widget=forms.PasswordInput(),
        error_messages={'current_admin_password': 'Please enter password'},
        required=False,
        help_text=_("This is the current password of the <b>assigned owner</b>")
    )

    portal_password = forms.CharField(widget=forms.PasswordInput(), required=False)
    confirm_portal_password = forms.CharField(widget=forms.PasswordInput(), required=False)
    cdr_password = forms.CharField(widget=forms.PasswordInput(), required=False)

    def _clean_hostname_field(self, field_name):
        host = self.cleaned_data[field_name].strip()
        HostnameValidator(host)
        return host

    def clean_portal_host(self):
        return self._clean_hostname_field('portal_host')

    def clean_cdr_host(self):
        return self._clean_hostname_field('cdr_host')

    def clean_replay_host(self):
        return self._clean_hostname_field('replay_host')

    def clean(self, *args, **kwargs):
        """
        Make sure that if the user is changing their password that the old
        password is correct and that the 2 copies of the new one match.
        """
        errs = list()
        data = super(VidyoAdminForm, self).clean(*args, **kwargs)

        # .get() because the field isn't always on the model
        current_admin_password = data.get('current_admin_password')
        if self.instance.pk:
            try:
                self.instance.decrypt(current_admin_password)
            except ValueError:
                errs.append(
                    ValidationError(
                        _("Password is incorrect."),
                        code='current_admin_password',
                        params={'current_admin_password': ''}
                    )
                )
        else:
            if 'portal_password' not in self.changed_data or not len(data.get('portal_password')):
                errs.append(
                    ValidationError(
                        _("The portal password is required to create the model."),
                        code='portal_password',
                        params={'portal_password': ''}
                    )
                )

        # Changing assigments is not allowed after instantiation.
        # If the POST data contains this field, then someone is doing something they shouldn't
        if self.instance.pk and 'dj_auth_user' in data:
            raise SuspiciousOperation(
                _("An attempt to modify an existing ForeignKey dj_auth_user was detected")
            )
        user = data.get('dj_auth_user')
        if not user and not self.instance.pk:
            errs.append(
                ValidationError(
                    _("You must assign a user"),
                    code='dj_auth_user',
                    params={'dj_auth_user': ''})
            )

        if 'portal_password' in self.changed_data or 'confirm_portal_password' in self.changed_data:
            if current_admin_password:
                user = user or self.instance.dj_auth_user
                if not user.check_password(current_admin_password):
                    errs.append(
                        ValidationError(
                            _("Password is incorrect."),
                            code='current_admin_password',
                            params={'current_admin_password': ''}
                        )
                    )
                portal_password = data['portal_password']
                confirm_portal_password = data['confirm_portal_password']

                if portal_password != confirm_portal_password:
                    errs.append(
                        ValidationError(
                            _("Passwords do not match."),
                            code='confirm_portal_password',
                            params={'confirm_portal_password': ''}
                        )
                    )
            else:
                errs.append(ValidationError(
                    _("Password required"),
                    code='current_admin_password',
                    params={'current_admin_password': ''})
                )
        if len(errs):
            raise ValidationError(errs)
        return self.cleaned_data


class VidyoAdminAdmin(ModelAdmin):
    """
    Custom ModelAdmin class which overrides things to allow the user to
    change their password while editing a VidyoAdmin object.
    """

    form = VidyoAdminForm
    inlines = (IPModelInline,)
    list_display = ('portal_user',
                    'portal_host',
                    'replay_host',
                    'portal_prefix',
                    'ssl',
                    'use_ip_auth',
                    'user_count',
                    )
    search_fields = ['portal_user']
    fieldsets = (
        (
            None,
            {
                'fields': (
                    'dj_auth_user',
                    'portal_host',
                    'portal_user',
                    'portal_prefix',
                    'ssl',
                    'use_ip_auth',
                    'user_proxy',
                    'user_group',
                    'user_location_tag',
                    'replay_host',
                )
            }
        ),
        (
            _("Call Detail Records options"),
            {
                'classes': ('collapse',),
                'fields': (
                    'cdr_enabled',
                    'cdr_port',
                    'cdr_host',
                    'cdr_user',
                    'cdr_password',
                )
            }
        ),
        (
            _("Portal API user password"),
            {
                'fields': (
                    'portal_password',
                    'confirm_portal_password',
                )
            }
        ),
        (_("Confirmation"), {'fields': ('current_admin_password',)}),
    )

    def get_readonly_fields(self, request, obj=None):
        if obj is not None and obj.pk:
            return ['dj_auth_user']
        return list()

    def save_model(self, request, obj, form, change):
        """
        Override this method to do the password changing as well as the
        normal object saving.
        """
        if change:
            if not self.has_change_permission(request):
                raise PermissionDenied()
        if obj.pk is None:
            if not self.has_add_permission(request):
                raise PermissionDenied()

        if form.cleaned_data.get('current_admin_password'):
            current_admin_password = form.cleaned_data['current_admin_password']
            portal_password = form.cleaned_data['portal_password']
            # If current_admin_password is set and we've got to here then the
            # we just need to check whether we're interested in updating the password.
            # If not, we still have to decrypt the admin object
            if portal_password:
                if change:
                    obj.change_password(current_admin_password, portal_password)
                else:
                    obj.set_password(current_admin_password, portal_password)
            else:
                obj.decrypt(current_admin_password)
        obj.save()

        return super(VidyoAdminAdmin, self).save_model(request, obj, form, change)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import logging

from django.conf import settings
from django.conf import ImproperlyConfigured

from . import default_settings

log = logging.getLogger(__name__)

__all__ = ['config']

PREFIX = 'OYDIV_'


class _Config(object):
    """
    Allow our app to have some preconfigured defaults, that are overriden
    in the django settings file.
    """

    def __getattr__(self, attr):
        try:
            return getattr(settings, PREFIX + attr)
        except AttributeError:
            try:
                return getattr(default_settings, attr)
            except AttributeError:
                raise ImproperlyConfigured(
                    "The setting '{}' must be defined in django settings.".format(PREFIX + attr)
                )

config = _Config()



# -*- coding: utf-8 -*-

from django.test import TestCase

from dj_oydiv.config import config


class DefaultConfigTests(TestCase):
    def test_overrides(self):
        """Ensure that settings are overriden in the config
        object when the user has defined the corresponding variable in
        settings.py
        """
        orig = config.DEFAULT_PROXY_NAME
        newsetting = orig * 2
        with self.settings(OYDIV_DEFAULT_PROXY_NAME=newsetting):
            self.assertEqual(config.DEFAULT_PROXY_NAME, newsetting)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import dj_oydiv.models
from django.conf import settings
import django.core.validators



try:
# Try and import the legacy application, and depend on its last migration which
# renames our db tables, from which this application takes over.
# If not, we are starting a new migration, and don't need the legacy cruft.
    import ajentavidyo
    dependencies = [
            ('ajentavidyo', '0008_final'),
            ('contenttypes', '0002_remove_content_type_name'),
            migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ]
except ImportError:
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]


class Migration(migrations.Migration):
    dependencies = dependencies
    operations = [
        migrations.CreateModel(
            name='IPModel',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('ip', models.GenericIPAddressField(help_text='Add the IP address of every authorised client', default='127.0.0.1', validators=[django.core.validators.validate_ipv46_address])),
            ],
        ),
        migrations.CreateModel(
            name='PasswordCrypt',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('ciphertext', models.TextField(editable=False, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
        ),
        migrations.CreateModel(
            name='VidyoAdmin',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('portal_user', models.CharField(help_text='The admin/operator user on the VidyoPortal', default='admin', verbose_name='API username', max_length=255)),
                ('portal_host', models.CharField(help_text='FQDN of the portal', default='example.com', verbose_name='VidyoPortal hostname', validators=[dj_oydiv.models._validate_hostname], max_length=65535)),
                ('portal_prefix', models.CharField(help_text='Tenant prefix (for multitenant portals)', default='001', verbose_name='Prefix', validators=[dj_oydiv.models._validate_prefix], max_length=4)),
                ('user_proxy', models.CharField(help_text='Default Proxy name for TCP tunneling of conferences', default='No Proxy', verbose_name='Default Proxy', max_length=128)),
                ('user_group', models.CharField(help_text='Default Group for new accounts. This is used to limit bandwidth and concurrency on the portal', default='Default', verbose_name='Default Group', max_length=128)),
                ('user_role', models.CharField(editable=False, default='Normal', choices=[('Normal', 'Normal'), ('Operator', 'Operator'), ('Admin', 'Admin')], max_length=128)),
                ('user_location_tag', models.CharField(help_text='Default location tag for new users (router affinity) formultiple router configuration.', default='Default', verbose_name='Location Tag', max_length=128)),
                ('replay_host', models.CharField(help_text="The FQDN of the given tenant's replay", default='example.com', verbose_name='VidyoReplay hostname', blank=True, validators=[dj_oydiv.models._validate_hostname], max_length=65535, null=True)),
                ('cdr_enabled', models.BooleanField(default=False, verbose_name='Allow collection of Call Detail Records')),
                ('cdr_port', models.PositiveIntegerField(default=3306, verbose_name='MySQL port for CDR access')),
                ('cdr_host', models.CharField(null=True, blank=True, verbose_name='MySQL hostname', validators=[dj_oydiv.models._validate_hostname], max_length=65535)),
                ('cdr_password', models.CharField(null=True, blank=True, verbose_name='MySQL password for CDR access', max_length=128)),
                ('cdr_user', models.CharField(default='cdraccess', verbose_name='MySQL user for CDR access', max_length=128)),
                ('use_ip_auth', models.BooleanField(default=True, verbose_name='Client IP whitelisting enabled')),
                ('ssl', models.BooleanField(default=True, verbose_name='SSL enabled VidyoPortal')),
                ('dj_auth_user', models.OneToOneField(help_text='The user to which this VidyoAdmin belongs.', null=True, verbose_name='Assigned Owner', to=settings.AUTH_USER_MODEL)),
            ],
            bases=(dj_oydiv.models.VidyoCredentialsMixin, models.Model),
        ),
        migrations.AddField(
            model_name='ipmodel',
            name='_admin_model',
            field=models.ForeignKey(to='dj_oydiv.VidyoAdmin'),
        ),
        migrations.AlterUniqueTogether(
            name='passwordcrypt',
            unique_together=set([('content_type', 'object_id')]),
        ),
    ]



from __future__ import absolute_import, unicode_literals

import logging
import functools

from django.contrib.auth import authenticate, login
from django.core.exceptions import ObjectDoesNotExist

from .basic import http_basic_credentials as basic_auth
from .basic import HttpResponseBasicChallenge

from ..utils.request import get_client_ip

logger = logging.getLogger(__name__)


def vidyo_admin_rpc_auth(view, auth_fn=basic_auth, fail_class=HttpResponseBasicChallenge):
    """django view decorator that attempts to authenticate an RPC call with
    the django standard login methods, and supplying the vidyo admin user as an
    attribute of the request object, and decrypt the admin's encrypted rpc
    credentials. Views will then be able to do the following.:

    >>> def myview(request)
        try:
            request.user.vidyoadmin.some_field
        except (AttributeError) as e:
            print("No Videos for you!:%r", e)

    `auth_fn` (if given) is expected to return a tuple of the (username, password)
    for the user.

    `fail_class` is any callable that returns an HttpResponse suitable for the
    failure. This could simply be `HttpResponeForbidden`, or similar. The only
    requirement is that it returns an HttpRespone.
    It is called with a single string argument of the reason for failure.
    """

    @functools.wraps(view)
    def fn(request, *args, **kwargs):
        try:
            username, password = auth_fn(request)
        except (AttributeError, ValueError):
            return fail_class("Forbidden")

        if not request.user.is_authenticated():
            logger.debug(" user '%s' was not authenticated", request.user)
            request.user = authenticate(username=username, password=password)
            if request.user is not None:
                if request.user.is_active:
                    login(request, request.user)
                else:
                    logger.debug(" user '%s' is not active", request.user)
                    return fail_class("Forbidden")
            else:
                logger.debug("Anonymous user was not authenticated", request.user)
                return fail_class("Forbidden")
        try:
            request.user.vidyoadmin.decrypt(password)
        except (ValueError, ObjectDoesNotExist):
            return fail_class("Forbidden")
        if not request.user.vidyoadmin.is_ip_allowed(get_client_ip(request)):
            logger.debug("Access from disallowed IP address:%s", get_client_ip(request))
            return fail_class("Forbidden")
        logger.info("Vidyo auth allowed for user '%s'", request.user)

        return view(request, *args, **kwargs)

    return fn



from __future__ import absolute_import, unicode_literals

import base64

from django.http import HttpResponse
from django.utils.encoding import force_bytes


class HttpResponseBasicChallenge(HttpResponse):
    status_code = 401

    def __init__(self, realm='Unknown'):
        super(HttpResponseBasicChallenge, self).__init__()
        self['WWW-Authenticate'] = 'Basic realm="%s"' % realm


def http_basic_credentials(request):
    """
    Strips the plaintext HTTP basic auth parameters out of the given request.
    Returns a bytestring tuple of (username, password)
    """
    try:
        # rfc2617 specifies the Authorization header only contains
        # 'Basic '+ base64 alphabet chars, which is a subset of utf-8
        auth_header = force_bytes(request.META['HTTP_AUTHORIZATION'])
    except KeyError:
        raise AttributeError("The request has no HTTP AUTH data")
    auth64 = auth_header.split(b'Basic ')[1]
    return base64.b64decode(auth64).split(b':', 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import logging
import copy

import arrow

from django.utils.functional import SimpleLazyObject
from django.core.exceptions import ObjectDoesNotExist

from oydiv_rpc import user
from oydiv_rpc import replay
from oydiv_rpc.member import PortalMember
from oydiv_rpc.exceptions import BaseError

# The entire API:
"""
vidyouser.conference_rpc.authentication_credentials(**kwargs)
vidyouser.conference_rpc.activate(endpoint_id)
vidyouser.conference_rpc.status()
vidyouser.conference_rpc.join_conference()
vidyouser.conference_rpc.exit_conference()

vidyouser.admin_rpc.create() # returns a unique integer id for the account
vidyouser.admin_rpc.delete() # returns ``None`` or raises an Exception
vidyouser.admin_rpc.update(**kwargs) # update the existing account details on the portal.

vidyouser.recording_rpc.start_recording()
vidyouser.recording_rpc.stop_recording()
vidyouser.recording_rpc.all()
vidyouser.recording_rpc.filter(**kwargs)
vidyouser.recording_rpc.get(id)
vidyouser.recording_rpc.delete(id)
"""

logger = logging.getLogger(__name__)


class RPCError(BaseError):
    pass


class Recording(object):
    def __init__(self, record, library_rpc):
        self.id = record.guid
        self.fp = library_rpc.open(record)
        self.start = arrow.get(record.dateCreated)
        self.end = arrow.get(record.endTime).datetime if record.endTime is not None else None
        self.file_bytes = record.fileSize

    def __len__(self):
        """
        Size of the recorded file in bytes.
        """
        return int(self.file_bytes)


class SimpleAdminRPCWrapper(object):
    def __init__(self, hostname, admin_username, admin_password, ssl, **kwargs):
        """
        We don't want to cause an unnecessary RPC requests if the object access
        does not actually require RPC. We defer this by using SimpleLazyObject.
        """
        member = PortalMember(**kwargs)

        self.rpc = SimpleLazyObject(
            lambda: user.User(
                hostname,
                user=admin_username,
                password=admin_password,
                api_type='admin',
                member=member,
                ssl=ssl,
            )
        )

    def create(self):
        return self.rpc.create()

    def delete(self):
        try:
            return self.rpc.delete()
        except user.DoesNotExist as e:
            raise ObjectDoesNotExist(str(e))

    def update(self, **kwargs):
        return self.rpc.update(**kwargs)


class SimpleUserRPCWrapper(object):
    def __init__(self, hostname, username, password, ssl, pin):
        self._pin = pin
        self.rpc = SimpleLazyObject(
            lambda: user.User(hostname, user=username, password=password, ssl=ssl)
        )

    def authenticate_endpoint(self, **kwargs):
        """
        Get information necessary to allow an endpoint to authenticate itself.
        All kwargs are passed to the underlying transport.
        """
        # We modify kwargs
        kwargs = copy.copy(kwargs)
        guest = kwargs.pop('guest', True)
        return self.rpc.endpoint_login_data(guest=guest, **kwargs)

    def activate_endpoint(self, endpoint_id, **kwargs):
        """
        Given an authenticated endpoint, activate it so that calls can be made.
        It is an error to attempt to activate an inactive client, but this is
        not checked.
        """
        return self.rpc.activate_endpoint(endpoint_id, **kwargs)

    def join_conference(self, conference, **kwargs):
        """
        Bring an activated endpoint into a call. It is an error to call this
        on an inactive endpoint.
        """
        if not isinstance(conference, (int, )):
            raise TypeError(
                "Conferences must be referenced by their integer id, or"
                "be a subclass of VidyoUserBase"
            )
        return self.rpc.join_room(None, room_id=conference, pin=self._pin, **kwargs)

    def terminate(self):
        """
        Eject the user's endpoints from all calls, and deauthenticate known
        endpoints.
        """
        self.rpc.log_out()


class SimpleReplayRPCWrapper(object):
    """
    A simple collection of methods for interacting with the given user's recording
    functionality (both control of recording, and lookup of past recordings)
    """

    def __init__(self, portal_hostname, replay_hostname, username, password, ssl, **kwargs):
        """
        As replay recording control and library access happen on two separate
        hosts, we need to create RPC backends for both the portal and the replay.
        The Lazy behaviour means we don't actually hit 2 servers unless such
        methods are called.
        """
        self.library_rpc = SimpleLazyObject(
            lambda: replay.Replay(
                replay_hostname,
                username,
                password,
                ssl=ssl
            )
        )
        self.rpc = SimpleLazyObject(
            lambda: user.User(portal_hostname, user=username, password=password)
        )

    def all(self):
        """ generator of all recordings ordered ascending by date"""
        return (Recording(x, self.library_rpc) for x in self.library_rpc.all_records)

    def get(self, id):
        return Recording(self.library_rpc.open(id), self.library_rpc)

    def filter(self, **kwargs):
        return (Recording(x, self.library_rpc) for x in self.library_rpc.filter(**kwargs))

    def start_recording(self, quality=-1):
        """Start recording the currently active user conference (if any).
        If the user is not in a conference raises LookupError."""
        self.rpc.start_recording(quality=quality)

    def stop_recording(self):
        """
        Stop any current recording for the user. Raises LookupError if no
        recording is in progress.
        Return a uuid suitable for later retrieval with ``get`` or
        `filter(uuid=uuid)``
        """
        return self.rpc.stop_recording()


class VidyoUserBaseSimpleRPCMixin(object):
    """
    Make rpc easy.
    This mixin is just a bunch of properties to abstract the access
    to the underlying RPC library without having to tediously build the RPC
    manually each time.
    """

    RPCError = RPCError

    # We can't use the lru_cache for admin because before save() we don't
    # have a pk, and that prevents the model from being hashable.
    # lru_cache requires hashable arguments
    @property
    def admin_rpc(self):
        return SimpleAdminRPCWrapper(
            self.admin_model.portal_host,
            self.admin_model.portal_user,
            self.admin_model.password,
            self.use_ssl(),
            # All kwargs are passed directly to PortalMember ctor
            name=self.portal_name,
            password=self.password,
            displayName=self.portal_displayname,
            Language=self.portal_language,
            groupName=self.portal_group,
            proxyName=self.portal_proxy,
            extension=self.portal_extension,
            emailAddress=self.portal_email,
            RoleName=self.portal_role,
            description=self.portal_description,
            allowCallDirect=self.can_call,
            allowPersonalMeeting=self.can_host,
            locationTag=self.portal_locationtag
        )

    @property
    def conference_rpc(self):
        return SimpleUserRPCWrapper(
            self.admin_model.portal_host,
            self.portal_name,
            self.password,
            self.use_ssl(),
            self.pin,
        )

    @property
    def recording_rpc(self):
        return SimpleReplayRPCWrapper(
            self.admin_model.portal_host,
            self.admin_model.replay_host,
            self.name,
            self.password,
            self.use_ssl()
        )



from __future__ import absolute_import, unicode_literals

import six

from django.test import TestCase

from .models import (
    TestVidyo1,
    VidyoAdmin,
)
from dj_oydiv.models import IPModel


class TestVidyoObjectDunderStr(TestCase):
    """
    All the Vidyo models have overridden ``__str__()`` methods. These testcases
    are just here for py2/py3 compatibility checks
    """
    def test_vidyoadmin_str(self):
        admin = VidyoAdmin()
        self.assertIn('VidyoAdmin', six.text_type(admin))
        self.assertIn('VidyoAdmin', str(admin))

    def test_vidyo_user_str(self):
        user = TestVidyo1(_admin_model=VidyoAdmin())
        self.assertIn('TestVidyo1', six.text_type(user))
        self.assertIn('TestVidyo1', str(user))

    def test_ipmodel_str(self):
        ip = IPModel(ip='[100::]')
        self.assertIn('[100::]', six.text_type(ip))
        self.assertIn('[100::]', str(ip))



from __future__ import absolute_import, unicode_literals

import logging
import itertools

from django.utils.translation import ugettext as _
from django.apps import apps
from django.db import models, transaction, IntegrityError
from django.conf import settings
from django.core import validators
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.views.decorators.debug import sensitive_variables
from django.utils import ipv6
from django.utils.decorators import method_decorator
from django.utils.encoding import force_text, force_bytes, python_2_unicode_compatible
from django.utils.timezone import now

from django.contrib.contenttypes.fields import (
    GenericForeignKey,
    GenericRelation,
    ContentType
)

from .utils.random import random_digits, random_string
from .utils.crypto import sym_decrypt_cfb_dict, sym_encrypt_cfb_128
from .rpc_wrapper import VidyoUserBaseSimpleRPCMixin
from .config import config

logger = logging.getLogger(__name__)


VIDYO_LANGUAGES = (
    ('en', _('English')),
    ('de', _('German')),
    ('es', _('Spanish')),
    ('fr', _('French')),
    ('it', _('Italian')),
    ('ja', _('Japanese')),
    ('ko', _('Korean')),
    ('pt', _('Portuguese')),
    ('zh_CN', _('Chinese (Simplified)')),
    ('fi', _('Finnish')),
    ('pl', _('Polish')),
    ('zh_TW', _('Taiwanese')),
    ('th', _('Thai')),
    ('ru', _('Russian'))
)


def _random_pin():
    return random_digits(11)


def _validate_prefix(prefix):
    if (not prefix.isdigit()) or int(prefix) < 1 or int(prefix) > 999:
        raise ValidationError()


def _validate_hostname(host):
    return validators.RegexValidator(
        validators.URLValidator.domain_re,
        message=_("Enter a valid hostname")
    )(host)


def _validate_pin(x):
    if (not x.isdigit()) or len(x) < 2 or len(x) > 11:
        raise ValidationError()


def _is_vidyouserbase(x):
    return issubclass(x, VidyoUserBase)


class CryptoText(models.Model):
    """
    This model stores short amounts of encrypted text.
    """

    ciphertext = models.TextField(editable=False, null=True)

    class Meta:
        abstract = True

    @method_decorator(sensitive_variables())
    def __init__(self, *args, **kwargs):
        self.key = kwargs.pop('key', None)
        data = kwargs.pop('data', None)
        super(CryptoText, self).__init__(*args, **kwargs)
        self.data = data

    @property
    @method_decorator(sensitive_variables())
    def data(self):
        """
        A text string of the decrypted data.
        """
        return force_text(self._decrypt(self.key))

    @data.setter
    @method_decorator(sensitive_variables())
    def data(self, text):
        if text is not None:
            try:
                self._encrypt(self.key, text)
            except AttributeError:
                raise ValueError('decryption failed: Key empty')

    @method_decorator(sensitive_variables())
    def _todict(self):
        """
        retrieve a dict suitable for decryption with sym_decrypt_cfb_128()
        """

        scheme, iv, kdf_iter, kdf_salt, hmac_algo, hmac_hex, ciphertext \
            = self.ciphertext.split("$")

        assert 'KDF:PBKDF2' in scheme
        assert 'CRYPT:AES-128-CFB' in scheme
        assert 'HMAC:SHA256' in scheme
        return {
            'ciphertext_64': force_bytes(ciphertext),
            'aes_iv_64': force_bytes(iv),
            'kdf_salt_64': force_bytes(kdf_salt),
            'kdf_iter': int(kdf_iter),
            'hmac_hex': force_bytes(hmac_hex),
            'hmac_algo': hmac_algo,
        }

    @method_decorator(sensitive_variables())
    def _decrypt(self, key):
        """
        Decrypt the stored data using the given key. Returns a raw bytestring.
        To access the original encoded text, use the ``data`` @property member.
        """
        if not self.ciphertext:
            return None
        try:
            return sym_decrypt_cfb_dict(key, self._todict())
        except:
            raise ValueError('decryption failed: wrong key')

    @method_decorator(sensitive_variables())
    def _encrypt(self, key, cleartext):
        """
        Encrypt the given cleartext
        """
        if len(key) < 8:
            raise ValueError("key too short")
        crypto = sym_encrypt_cfb_128(key, cleartext)
        self.ciphertext = "$".join(map(force_text, (
            'CRYPT:AES-128-CFB_HMAC:SHA256_KDF:PBKDF2',
            crypto['aes_iv_64'],
            crypto['kdf_iter'],
            crypto['kdf_salt_64'],
            'sha256',
            crypto['hmac_hex'],
            crypto['ciphertext_64'],
        )))

    @method_decorator(sensitive_variables())
    def change_key(self, newkey):
        """
        Given a newkey, reencrypts the data with the new key.
        requires @self.key already instantiated
        """
        self._encrypt(newkey, self.data)
        self.key = newkey

    def verify(self):
        # TODO
        pass

    def save(self, *args, **kwargs):
        self.verify()
        super(CryptoText, self).save()


class PasswordCrypt(CryptoText):
    """
    An emulated generic OneToOneField allowing one password per anykind of Model.
    """
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        unique_together = ('content_type', 'object_id')


class VidyoCredentialsMixin(object):
    """
    This mixin abstracts many of the encrypted VidyoPassword get/set
    functionality for all VidyoUser/VidyoAdmin classes.
    """
    @property
    @method_decorator(sensitive_variables())
    def _password_model(self):
        try:
            generic_pass = self._passwordcrypt.get()
        except self._passwordcrypt.model.DoesNotExist:
            return self._passwordcrypt.create()
        return generic_pass

    @property
    @method_decorator(sensitive_variables())
    def password(self):
        try:
            return self._password
        except AttributeError:
            return self.get_password(self.secret_key)

    @password.setter
    @method_decorator(sensitive_variables())
    def password(self, val):
        self.set_password(self.secret_key, val)

    @method_decorator(sensitive_variables())
    def get_password(self, key):
        self.secret_key = key
        try:
            return self._password_model.data
        except ValueError:
            self.secret_key = key
            cryptopass = self._password_model
            cryptopass.key = key
            self._password = cryptopass.data
        return self._password

    @method_decorator(sensitive_variables())
    def decrypt(self, key):
        self.secret_key = key
        self.get_password(key)
        return self

    @method_decorator(sensitive_variables())
    def change_key(self, old_key, new_key):
        if isinstance(self, VidyoAdmin):
            logger.warn("changing all keys for %r", self)
            self._change_all_keys(old_key, new_key)
        try:
            self._password_model.data
        except ValueError:
            self.secret_key = old_key
            cryptopass = self._password_model
            cryptopass.key = old_key
            cryptopass.change_key(new_key)
            cryptopass.save(commit=False)

    @method_decorator(sensitive_variables())
    def set_password(self, auth_user_key, vidyo_password=None):
        self.secret_key = auth_user_key
        if vidyo_password is not None:
            self._password = vidyo_password
        #no password has been set. We have to make sure we have a pk before
        #assigment so saving is deferred to model save.  In that case,
        #if a model is discarded, the password is discarded with it.
        return self

    @method_decorator(sensitive_variables())
    def change_password(self, old_auth_user_key, new_key):
        self.get_password(old_auth_user_key)
        self.password = new_key

    @method_decorator(sensitive_variables())
    def _save_password(self):
        cryptopass = self._password_model
        cryptopass.key = self.secret_key
        try:
            cryptopass.data = self._password
        except AttributeError:
            cryptopass.data = None
        cryptopass.save()

    @method_decorator(sensitive_variables())
    def _change_all_keys(self, old_key, new_key):
        """
        Changing the password is a dangerous and expensive operation, as the
        decryption/encryption needs to be handled for all children.
        We wrap the updates in an atomic transaction, so that no vidyo users can
        be created/modified until this transaction completes.
        This method is part of the mixin to avoid having to overload the admin's
        save() method, and avoid any MRO issues/recursive save() calls etc.
        """
        if not isinstance(self, VidyoAdmin):
            raise TypeError("%s must be called from a VidyoAdmin object")
        with transaction.atomic():
            for child in self.all_vidyo_users():
                child.change_key(old_key, new_key)

    def save(self, *args, **kwargs):
        super(VidyoCredentialsMixin, self).save(*args, **kwargs)
        self._save_password()


@python_2_unicode_compatible
class VidyoAdmin(VidyoCredentialsMixin, models.Model):
    dj_auth_user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        null=True,
        help_text=_("The user to which this VidyoAdmin belongs."),
        verbose_name=_("Assigned Owner")
    )
    _passwordcrypt = GenericRelation(PasswordCrypt)

    #Portal Specific
    portal_user = models.CharField(
        _("API username"),
        help_text=_("The admin/operator user on the VidyoPortal"),
        max_length=255,
        default='admin'
    )
    portal_host = models.CharField(
        _("VidyoPortal hostname"),
        help_text=_("FQDN of the portal"),
        max_length=65535,
        default='example.com',
        validators=[_validate_hostname]
    )
    portal_prefix = models.CharField(
        _("Prefix"),
        help_text=_("Tenant prefix (for multitenant portals)"),
        max_length=4, default='001',
        validators=[_validate_prefix]
    )
    user_proxy = models.CharField(
        _("Default Proxy"),
        help_text=_("Default Proxy name for TCP tunneling of conferences"),
        max_length=128, default=config.DEFAULT_PROXY_NAME
    )
    user_group = models.CharField(
        _("Default Group"),
        help_text=_(
            "Default Group for new accounts. "
            "This is used to limit bandwidth and concurrency on the portal"),
        max_length=128, default=config.DEFAULT_GROUP_NAME
    )
    user_role = models.CharField(
        max_length=128,
        choices=(
            ('Normal', _("Normal")),
            ('Operator', _("Operator")),
            ('Admin', _("Admin"))
        ),
        default=config.DEFAULT_ROLE_NAME,
        editable=False
    )
    user_location_tag = models.CharField(
        _("Location Tag"),
        help_text=_(
            "Default location tag for new users (router affinity)"
            " formultiple router configuration."),
        max_length=128, default=config.DEFAULT_LOCATION_TAG
    )

    # Replay specific
    replay_host = models.CharField(
        _("VidyoReplay hostname"),
        help_text=_("The FQDN of the given tenant's replay"),
        max_length=65535,
        default='example.com',
        null=True,
        blank=True,
        validators=[_validate_hostname]
    )

    # CDR specific
    cdr_enabled = models.BooleanField(
        _("Allow collection of Call Detail Records"),
        default=False
    )
    cdr_port = models.PositiveIntegerField(
        _("MySQL port for CDR access"),
        default=3306
    )
    cdr_host = models.CharField(
        _("MySQL hostname"),
        max_length=65535,
        null=True,
        blank=True,
        validators=[_validate_hostname]
    )
    cdr_password = models.CharField(
        _("MySQL password for CDR access"),
        max_length=128,
        null=True,
        blank=True
    )
    cdr_user = models.CharField(
        _("MySQL user for CDR access"),
        max_length=128,
        default='cdraccess'
    )

    # Authentication
    use_ip_auth = models.BooleanField(_("Client IP whitelisting enabled"), default=True)
    ssl = models.BooleanField(_("SSL enabled VidyoPortal"), default=True)

    def use_ssl(self):
        return self.ssl

    def allowed_ips(self):
        return self.ipmodel_set.iterator()

    def allowed_ips_strings(self):
        return [x.ip for x in self.allowed_ips()]

    def _allowed_cleaned_ip(self, ip):
        try:
            self.ipmodel_set.get(ip=ip)
        except self.ipmodel_set.model.DoesNotExist:
            return False
        return True

    def is_ip_allowed(self, ip):
        if not self.use_ip_auth:
            return True
        try:
            validators.validate_ipv46_address(ip)
        except ValidationError:
            return False
        if ipv6.is_valid_ipv6_address(ip):
            ip = ipv6.clean_ipv6_address(ip)
        return self._allowed_cleaned_ip(ip)

    def all_vidyo_users(self):
        """
        Get a generator that yields all the descendants of ``VidyoUserBase``
        owned by this admin.
        """
        return itertools.chain.from_iterable(
            T.objects.filter(_admin_model=self)
            for T in VidyoUserBase.all_vidyo_user_classes()
        )

    @property
    def user_count(self):
        """
        Number of VidyoUserBase descendants with this model as their parent admin.
        """
        return sum(
            T.objects.filter(_admin_model=self).count()
            for T in VidyoUserBase.all_vidyo_user_classes()
        )

    def __str__(self):
        return 'VidyoAdmin portal user %s' % self.dj_auth_user


@python_2_unicode_compatible
class IPModel(models.Model):
    """IP address foreignKey model for IP address whitelisting."""
    #This model exists so we can include a *list* of ip addresses in
    ip = models.GenericIPAddressField(
        default='127.0.0.1',
        validators=[validators.validate_ipv46_address],
        help_text=_("Add the IP address of every authorised client")
    )
    _admin_model = models.ForeignKey(VidyoAdmin)

    def save(self, *args, **kwargs):
        validators.validate_ipv46_address(self.ip)
        if ipv6.is_valid_ipv6_address(self.ip):
            self.ip = ipv6.clean_ipv6_address(self.ip)
        super(IPModel, self).save(*args, **kwargs)
        #if the ip address is ipv6, we want it sanitized

    def __str__(self):
        return str(self.ip)


@python_2_unicode_compatible
class VidyoUserBase(VidyoUserBaseSimpleRPCMixin, VidyoCredentialsMixin, models.Model):
    """
    Abstract base class mapping to a user account on the portal.
    This class abstracts many of the properties of the VidyoPortal admin/user API
    and offers convenience methods to allow easy access to RPC.

    To use this model, subclass in your app and add the following required fields::

        `client_id`: Anything you want to refer to this model. e.g. integer
        primary-key
        UUID, some string.

    Then override then following methods::

        `normalize_client_id(); staticmethod`: given your ``client_id``, sanitize it,
        and normalize the format for storing in the database.
        `get_name()`: return a unique name for the user as a string. This value is
            used directly on the portal as the account name so it should be unique.
            For example, If you're allowing users to manage their own accounts, you
            could simply return ``firstname.lastname``. For any situation in which
            you're managing the accounts automatically, a guaranteed unique value is
            probably better e.g. UUID string/hash over all credentials/random string.

        `get_extn()`: expected to return an extension for your model during save.
            Most of the time your can simply return a random number, which will
            be appended to the portal extension prefix before save.
    """
    _admin_model = models.ForeignKey(VidyoAdmin)
    _passwordcrypt = GenericRelation(PasswordCrypt)
    vidyo_entity_id = models.PositiveIntegerField(editable=False, null=True)

    created = models.DateTimeField(default=now)
    # All of the `portal_*` attributes are needed to create a member on the portal
    # see `oydiv_rpc.member.PortalMember` for gory implementation details.
    portal_name = models.CharField(max_length=255, unique=True, editable=False)
    portal_displayname = models.CharField(_("Full Name"), max_length=255)
    portal_language = models.CharField(max_length=2, choices=VIDYO_LANGUAGES, default='en')
    portal_group = models.CharField(
        max_length=255, default=config.DEFAULT_GROUP_NAME
    )
    portal_proxy = models.CharField(
        max_length=128, default=config.DEFAULT_PROXY_NAME
    )
    portal_extension = models.CharField(max_length=16, editable=False)
    portal_email = models.EmailField(
        default=config.DEFAULT_MANAGER_EMAIL,
        editable=False
    )
    portal_role = models.CharField(
        max_length=255,
        default=config.DEFAULT_ROLE_NAME,
        editable=False
    )
    portal_description = models.CharField(max_length=1024, editable=False)
    portal_locationtag = models.CharField(
        max_length=255,
        default=config.DEFAULT_LOCATION_TAG,
        editable=False
    )

    pin = models.CharField(
        editable=False,
        max_length=11,
        validators=[_validate_pin],
        default=_random_pin
    )
    # Permissions
    can_call = models.BooleanField(default=False, editable=False)
    can_host = models.BooleanField(default=True, editable=False)
    can_record = models.BooleanField(default=False, editable=False)

    class Meta:
        abstract = True

    @classmethod
    def all_vidyo_user_classes(cls):
        """List all django models that are concrete implementations of this abstract
        base class"""
        return filter(lambda x: issubclass(x, VidyoUserBase), apps.get_models())

    @property
    @method_decorator(sensitive_variables())
    def admin_model(self):
        model = self._admin_model
        try:
            model.decrypt(self.secret_key)
        except AttributeError:
            pass
        return model

    def use_ssl(self):
        return self.admin_model.use_ssl()

    def get_extn_prefix(self):
        """
        Get prefix used for all extensions.
        Unique extensions are *required* by the portal. They must also be
        prefixed by a tenant-global prefix.
        """
        return str(self.admin_model.portal_prefix) + str(self.extn_prefix)

    def get_extn(self):
        """Abstract method expected to be implemented by the client model"""
        raise NotImplementedError()

    def get_name(self):
        """Abstract method expected to be implemented by the client model"""
        raise NotImplementedError()

    @staticmethod
    def normalize_client_id(client_id):
        """Abstract method expected to be implemented by the client model
        This method is used to sanitize the client_id before it is saved in
        the database, and before retrieval, to avoid mismatches caused by
        different representations of the same data. For example, the following
        uuids are identical:
            {000d662f75524339b127bc75d8cd9514}
            000d662f-7552-4339-b127-bc75d8cd9514
        but a database is probably not aware of that.
        """
        return client_id

    @method_decorator(sensitive_variables())
    def default_setup(self):
        """
        This method is used in ``self.``self.save() to set default values
        assigned for the tenant, or overriden on the admin.
        Only values that make sense to be inherited as defaults are inherited.
        This method only exists because Vidyo's API gives no way to query this
        information, and so we treat it as a config value with
        ``_admin_model`` as the source of the config data. It's very ugly.
        """
        if self.pk is None and self._admin_model:
            self.portal_proxy = self.admin_model.user_proxy
            self.portal_group = self.admin_model.user_group
            self.portal_role = self.admin_model.user_role
            # Now set the unique user values
            self.portal_name = self.get_name()
            self.portal_description = self.portal_name
            self.portal_extension = str(self.get_extn())
            self.portal_displayname = self.portal_displayname or _("Anonymous")
            try:
                self.password
            except (ObjectDoesNotExist, IntegrityError):
                self.password = random_string()

    def save(self, *args, **kwargs):
        if self.pk is None:
            self.default_setup()
            try:
                self.vidyo_entity_id = self.admin_rpc.create()
            except self.RPCError as e:
                logger.exception("RPC failed; couldn't create account on portal:%s", str(e))
                # This needs to be vague, because we don't know what sensitive
                # info might be in the portal's exception message.
                raise ValidationError(_("The remote portal could not verify the given data"))
        return super(VidyoUserBase, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        try:
            self.admin_rpc.delete()
        except ObjectDoesNotExist as e:
            # That's actually what we're aiming for, so just ignore it.
            # (Though it's a little fishy so log it)
            logger.exception("Couldn't delete VidyoUser; not found on portal: %s - %s", self, e)
        return super(VidyoUserBase, self).delete(*args, **kwargs)

    def __str__(self):
        return (
            '<%s: %s(%s)> extension:%s client_id:%s entity_id:%s' % (
                self.__class__, self.portal_name, self.portal_displayname,
                self.portal_extension, self.client_id, self.vidyo_entity_id
            )
        )



from __future__ import absolute_import, unicode_literals

from django.contrib.admin import site

from .override_password_form import monkeypatch_password_form
from .vidyoadmin import VidyoAdmin, VidyoAdminAdmin
from .vidyouser_generic import register_all_vidyo_models


monkeypatch_password_form()
register_all_vidyo_models()
site.register(VidyoAdmin, VidyoAdminAdmin)



from __future__ import absolute_import, unicode_literals
import os

import mock

from django.test import TestCase
from django.utils.encoding import force_text

from dj_oydiv.utils.random import random_ascii_letters
from dj_oydiv.models import VidyoAdmin
from dj_oydiv.utils.crypto import (
    sym_encrypt_cfb_128,
    sym_decrypt_cfb_dict
)
from . import PACKAGE_BASE
from .models import (
    CryptoTextTest,
    save_then_retrieve,
)


class TestCryptoTextTests(TestCase):
    """
    The test in this class validate basic sanity of the crypto object.
    These tests include sanity-checking/data verification that the data stored
    looks *somewhat* encrypted. They're mostly just functionality tests, and
    assertions that bad input results in errors as we're relying on the sanity
    on pyCrypto
    """
    cleartext = 'aaaaaaaaaaaaaaaaaa'
    key = 'password'
    badkey = 'wrong!'
    newkey = 'anothertotallysecretpassword'

    def test_no_cleartext_saved(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        t.save()
        pk = t.pk
        t = CryptoTextTest.objects.get(pk=pk)
        t.key = self.key
        self.assertNotEqual(t.ciphertext, self.cleartext)
        self.assertNotIn(self.cleartext, t.ciphertext)

    def test_change_key(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        t = save_then_retrieve(t, key=self.key)
        t.change_key(self.newkey)
        t = save_then_retrieve(t, key=self.newkey)
        self.assertEqual(self.cleartext, t.data)

    def test_data_property(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        self.assertNotEqual(t.data, t.ciphertext)
        old_ciphertext = t.ciphertext
        t.data = self.cleartext + self.cleartext
        self.assertNotEqual(old_ciphertext, t.ciphertext)

    def test_empty_key_fails(self):
        with self.assertRaises(ValueError):
            CryptoTextTest(data=self.cleartext, key='')

    def test_db_manager_search_with_kwargs_key(self):
        pass

    def test_verify_returned_data_equal(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        t = save_then_retrieve(t, key=self.key)
        self.assertEqual(self.cleartext, t.data)

    def test_bad_password_fails(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext

        with self.assertRaises(ValueError):
            t = CryptoTextTest()
            t.key = self.key
            t.data = self.cleartext
            save_then_retrieve(t, key=self.badkey).data

    def test_tamper_evident(self):
        """change a digit of the hmac_digest, and assert that decryption fails with ValueError"""
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        t = save_then_retrieve(t, key=self.key)
        scheme, iv, kdf_iter, kdf_salt, hmac_algo, hmac_hex, \
            ciphertext = t.ciphertext.split("$")

        l = int(hmac_hex, base=16)
        #convert the hmac to int, add 1, and convert back to hex
        hmac_hex = hex(l + 1)[2:-1]
        t.ciphertext = '$'.join((
            scheme, iv, kdf_iter, kdf_salt,
            hmac_algo, hmac_hex, ciphertext)
        )
        with self.assertRaises(ValueError):
            t.data

    def test_large_binary_data(self):
        """
        We're using a text field and require the ability to store large amount of binary data.
        """
        data = os.urandom(2 ** 20)
        t = CryptoTextTest(data=data, key=self.key)
        t = save_then_retrieve(t, key=self.key)
        self.assertEqual(t._decrypt(self.key), data)

    def test_same_params_different_outputs(self):
        """Verify we're using unique values for salt"""
        t1 = CryptoTextTest(data=self.cleartext, key=self.key)
        t2 = CryptoTextTest(data=self.cleartext, key=self.key)
        t1 = save_then_retrieve(t1, key=self.key)
        t2 = save_then_retrieve(t2, key=self.key)
        self.assertNotEqual(t1.ciphertext, t2.ciphertext)

    def test_crypto_roundtrip(self):
        t = CryptoTextTest()
        t.key = self.key
        t.data = self.cleartext
        t = save_then_retrieve(t, key=self.key)
        self.assertEqual(self.cleartext, t.data)


class TestMultipleCrypto(TestCase):
    """
    User objects delegate the responsiblity for handling re-keying to the assigned admin
    objects.
    Tests here check that Admin key-rotation works
    """
    def test_it_please(self):
        # XXX Refactor this into an actual test.
        for admin in [VidyoAdmin(portal_user=str(1 + x)) for x in range(1)]:
            admin.set_password('mypassword', 'mysecretdata')
            admin.save()


class TestRandomAsciiLetters(TestCase):

    ascii_printable = ''.join(chr(x) for x in range(0x20, 0x7f, 1))

    def test_issue81_regression(self):
        """We switched to django.utils.crypto.get_random_string() because we were
        generating passwords with the builtin mersenne twister, seeded manually
        from '/dev/urandom'.
        This had already been 'fixed' once due to missing parens in a seed() call.
        This regression test ensures that we're using django's sane, portable
        crypto-quality generator, instead of some unvetted pile.
        """
        with mock.patch(
            PACKAGE_BASE + '.utils.random.get_random_string',
            mock.MagicMock(return_value='thisisrandom')
        ) as patched:
            string = random_ascii_letters()
            self.assertEqual(string, 'thisisrandom')
            self.assertTrue(patched.called)

    def test_ascii_only(self):
        for x in range(1000):
            s = random_ascii_letters()
            for c in s:
                self.assertIn(c, self.ascii_printable)

    def test_multiple_iterations_nequal(self):
        """Totally naive basic randomness check.
        (the implementation uses django.utils.crypto.get_random_string()
        under the hood so this should suffice)
        """
        s1 = random_ascii_letters()
        s2 = random_ascii_letters()
        self.assertNotEqual(s1, s2)

    def test_crypto_dict_roundtrip(self):
        crypt = sym_encrypt_cfb_128('secret', 'sauce')
        self.assertEqual(force_text(sym_decrypt_cfb_dict('secret', crypt)), 'sauce')


class CryptoBackwardCompatibilityTests(TestCase):
    """
    We want the ability to add/remove dependencies as needed for speed, platform
    availability or similiar.
    However, data in production databases should still be valid.
    Test these fixtures decrypt to the expected values.
    A new feature, primitive or significant change should add a
    backwards compatibility test here
    """
    def test_backward_compatibility_sym_decrypt_cfb_128(self):
        # sym_encrypt_cfb_128('secret', 'something nobody knows') git@f6688dde6
        crypt = {
            'aes_iv_64': b'azm8m1zKYDQ/uuCGzqxMGA==',
            'ciphertext_64': b'1n3rGiH4AXBQyz4EGT6+0ruhf3ev9A==',
            'hmac_algo': 'sha256',
            'hmac_hex': 'ef849dedefc26b45c1d13cd50de5487159f9af9fccf987318217491f47287588',
            'kdf_algo': 'PBKDF2',
            'kdf_iter': 1,
            'kdf_salt_64': b'J0E0RKe0r+w='
        }
        self.assertEqual(
            force_text(sym_decrypt_cfb_dict('secret', crypt)),
            'something nobody knows'
        )



from __future__ import absolute_import, unicode_literals

import random
import base64

from django.test import TestCase
from django.core.validators import ValidationError
from django.http import HttpRequest, HttpResponse
from django.test.client import RequestFactory
from django.contrib.auth.models import User, AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.utils.encoding import force_bytes

from . import VIDYO_PORTAL_PASSWORD
from .models import create_test_admin

from dj_oydiv.auth.decorators import vidyo_admin_rpc_auth
from dj_oydiv.auth.basic import http_basic_credentials


class HttpBasicAuthTests(TestCase):
    USER = 'mac'
    PASSWORD = 'mygoodpassword'
    CLIENT_IP = '192.168.1.100'

    def _bare_credentialed_request(self, ip=None, user=None, password=None):
        extra = {}
        user = user or self.USER
        password = password or self.PASSWORD
        auth_header = b'Basic ' + base64.b64encode(force_bytes('%s:%s' % (user, password)))
        extra['HTTP_AUTHORIZATION'] = auth_header
        if ip:
            extra['REMOTE_ADDR'] = ip

        request = self.factory.get('/', **extra)
        return request

    def _session_middleware_credentialed_request(self, **kwargs):
        request = self._bare_credentialed_request(**kwargs)
        request.user = AnonymousUser()
        # setup sessions, etc.
        SessionMiddleware().process_request(request)

        return request

    def _good_credentials(self, ip=None, user=None, password=None):
        return self._session_middleware_credentialed_request(ip=ip, user=user, password=password)

    def _bad_credentials(self, ip=None, user=None, password=None):
        password = password or 'mybadpassword'
        return self._session_middleware_credentialed_request(ip=ip, user=user, password=password)

    def setUp(self):
        self.user = User.objects.create_user(
            username=self.USER, email='mac@example.com', password=self.PASSWORD
        )
        self.vidyo_admin = create_test_admin(
            dj_auth_user=self.user, password=self.PASSWORD
        )
        self.vidyo_admin.use_ip_auth = False
        self.vidyo_admin.save()
        self.factory = RequestFactory()

    def myview(self, request):
        """Just a dummy view to pass to the decorators"""
        return HttpResponse('success')

    def test_bad_credentials(self):
        request = self._bad_credentials()
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.content, b'')

    def test_no_credentials_401(self):
        response = vidyo_admin_rpc_auth(self.myview)(HttpRequest())
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.content, b'')

    def test_good_credentials_deny_empty_ip(self):
        self.vidyo_admin.use_ip_auth = True
        self.vidyo_admin.save()
        request = self._good_credentials()
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.content, b'')

    def test_good_credentials_good_ip(self):
        self.vidyo_admin.use_ip_auth = True
        self.vidyo_admin.save()
        self.vidyo_admin.ipmodel_set.create(ip=self.CLIENT_IP)
        request = self._good_credentials(ip=self.CLIENT_IP)

        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'success')

    def test_good_credentials_bad_ip(self):
        self.vidyo_admin.use_ip_auth = True
        self.vidyo_admin.save()
        self.vidyo_admin.ipmodel_set.create(ip=self.CLIENT_IP)
        request = self._good_credentials(ip='255.255.255.255')
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.content, b'')

    def test_good_credentials_no_ip_auth(self):
        request = self._good_credentials()
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'success')

    def test_successful_auth_user_login(self):
        """
        Tests for successful user login on delivery of good auth credentials
        The purpose of this is to avoid expensive calls to
        ``django.contrib.auth.authenticate()`` with every request, when
        a session can be set up instead.  (cookie support required clientside)
        """
        request = self._good_credentials()
        self.assertEqual(request.user.is_authenticated(), False)
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'success')
        self.assertEqual(request.user.is_authenticated(), True)
        self.assertEqual(self.user, request.user)

    def test_vidyo_admin_model_decrypted(self):
        """
        tests that given proper credentials, a decrypted vidyoadmin model is
        available as an attribute of request.user
        """
        request = self._good_credentials()
        vidyo_admin_rpc_auth(self.myview)(request)
        try:
            self.assertEqual(request.user.vidyoadmin.password, VIDYO_PORTAL_PASSWORD)
        except ValueError as e:
            self.addFailure(e)

    def test_user_no_vidyoadmin_401(self):
        """
        If a given user doesn't have a vidyoadmin FK, then authentication
        should fail, regardless  of the quality of the given credentials
        """
        User.objects.create_user(
            username='mac2', email='mac2@example.com', password='mymac2password'
        )
        request = self._session_middleware_credentialed_request(
            user='mac2', password='mymac2password'
        )
        response = vidyo_admin_rpc_auth(self.myview)(request)
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.content, b'')

    def test_extract_http_basic_credentials(self):
        user, password = http_basic_credentials(
            self._session_middleware_credentialed_request(user='silly', password='sausage')
        )
        self.assertEqual(user, b'silly')
        self.assertEqual(password, b'sausage')

    def test_extract_http_basic_credentials_with_colon(self):
        user, password = http_basic_credentials(
            self._session_middleware_credentialed_request(user='silly', password='sau:sage')
        )
        self.assertEqual(user, b'silly')
        self.assertEqual(password, b'sau:sage')


class IPAddressWhitelistTest(TestCase):
    ITERATIONS = 20

    def setUp(self):
        self.admin = create_test_admin()
        self.admin.use_ip_auth = True
        self.admin.save()

    @staticmethod
    def random_bad_ipv4():
        return ''.join(str(random.randint(256, 999)) + '.' for x in range(4))[:-1]

    @staticmethod
    def random_ipv4():
        return ''.join(str(random.randint(0, 255)) + '.' for x in range(4))[:-1]

    @staticmethod
    def random_ipv6():
        return ''.join(hex(random.randint(0, 65535))[2:] + ':' for x in range(8))[:-1]

    def test_ip6_multiple_representation(self):
        """
        IPv6 addresses can be formatted in multiple styles, and are often shortened
        with multiple colons when a particular tuple is all-zeros.
        This test checks that IP addresses are properly normalised before comparison.
        """
        self.admin.ipmodel_set.create(ip='0000:0000:0000:0000:0000:0000:0000:0001')
        self.assertTrue(self.admin.is_ip_allowed('::1'))

    def test_ip_auth_off_permissive(self):
        """
        Ensure that a reasonable set of random ip addresses
        are permitted when ip auth is off.
        """
        self.admin.use_ip_auth = False
        for x in range(self.ITERATIONS):
            self.assertTrue(self.admin.is_ip_allowed(self.random_ipv4()))
            self.assertTrue(self.admin.is_ip_allowed(self.random_ipv6()))

    def test_ip_auth_on_strict(self):
        """
        Make a reasonable claim that all ip addresses but
        those added to the allowed whitelist are rejected.
        """
        for x in range(self.ITERATIONS):
            self.assertFalse(self.admin.is_ip_allowed(self.random_ipv4()))
            self.assertFalse(self.admin.is_ip_allowed(self.random_ipv6()))

    def test_added_ip_passed(self):
        """check that any ip is rejected, until it is added"""
        ips = [self.random_ipv4() for x in range(self.ITERATIONS)] + \
              [self.random_ipv6() for x in range(self.ITERATIONS)]
        for ip in ips:
            self.assertFalse(self.admin.is_ip_allowed(ip))
            self.admin.ipmodel_set.create(ip=ip)
            self.assertTrue(self.admin.is_ip_allowed(ip))

    def test_malformed_ip_rejected(self):
        for x in range(self.ITERATIONS):
            with self.assertRaises(ValidationError):
                self.admin.ipmodel_set.create(ip=self.random_bad_ipv4())
                self.admin.save()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from importlib import import_module as _import


"""
The settings here are all sensible defaults, but can be overriden in settings.py
"""

# Allow the django project to decide on sensible defaults for crypto
CRYPTO_KDF_ITERATIONS = _import('django.contrib.auth.hashers').PBKDF2PasswordHasher.iterations
DEFAULT_PASSWORD_LENGTH = 32

# Defaults used in creating members on the portal.
DEFAULT_GROUP_NAME = 'Default'
DEFAULT_ROLE_NAME = 'Normal'
DEFAULT_PROXY_NAME = 'No Proxy'
DEFAULT_LOCATION_TAG = 'Default'
# The portal will reject this email. It must overridden.
DEFAULT_MANAGER_EMAIL = 'root@localhost'

# dictionary of passwords keyed by username used to authenticate a portal
# attemping webservices authentication.
# see views.wsauth.
WSAUTH_PORTAL_CREDENTIALS = {}



from __future__ import absolute_import, unicode_literals

import base64
import hashlib
import hmac
import logging

from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes

from django.utils.encoding import force_bytes

from ..config import config

logger = logging.getLogger(__name__)

ITERATIONS = config.CRYPTO_KDF_ITERATIONS


try:
    constant_time_compare = hashlib.compare_digest
except AttributeError:
    # python < 2.7.9; See [PEP0466](https://www.python.org/dev/peps/pep-0466/)
    from django.utils.crypto import constant_time_compare

try:
    from hashlib import pbkdf2_hmac
    pbkdf2_derive_key = pbkdf2_hmac
    logger.info("Using hashlib.pbkdf2_hmac for key derivation")
except ImportError:
    # python < 2.7.9; See [PEP0466](https://www.python.org/dev/peps/pep-0466/)
    logger.warning("PEP0466 not supported. Falling back to django pbkdf2")
    from django.utils.crypto import pbkdf2

    def pbkdf2_derive_key(hash_name, password, salt, iterations, dklen):
        return pbkdf2(password, salt, iterations, dklen, getattr(hashlib, hash_name))


def sym_encrypt_cfb_128(secret_key, cleartext, hmac_algo='sha256', kdf_iterations=ITERATIONS):
    """perform AES-128 symmetric encryption on an arbitrary cleartext
    using the given secret_key in CFB mode.

    AES-CFB does not require padding the input, so simplifies the task a bit.
    returns a dict of the base64-encoded ciphertext, base64-encoded IV,
    the HMAC-digest-hex, the HMAC-digest-algorithm name,
    the key-derivation function used/number of rounds.
    presently, PBKDF2 is used for key-derivation.

    """
    secret_key = force_bytes(secret_key)
    cleartext = force_bytes(cleartext)
    assert hmac_algo in getattr(
        hashlib,
        'algorithms_available', # See [PEP0466](https://www.python.org/dev/peps/pep-0466/)
        getattr(hashlib, 'algorithms', [])
    )

    # Generate the AES initialisation vector, and the salt used by the key-derivation algo
    salt = get_random_bytes(8)
    iv = get_random_bytes(16)

    # AES-CFB-128 needs a 16byte key.
    # Use PBKDF2 to generate a 16byte key regardless of the secret_key length.
    key = pbkdf2_derive_key('sha1', secret_key, salt, kdf_iterations, 16)

    # we're using AES-128 because Bruce Schneier says not to use 192/256.
    # Who are we to argue?
    # https://www.schneier.com/blog/archives/2009/07/another_new_aes.html#c386957
    crypt = AES.new(key, mode=AES.MODE_CFB, IV=iv)
    ciphertext_64 = base64.b64encode(crypt.encrypt(cleartext))

    # generate the Message Authentication Code according to @hmac_algo
    # The MAC function is passed the cleartext + iv, and password.
    # use the IV to avoid the same cleartext / pw combination giving the same MAC
    hash_fn = getattr(hashlib, hmac_algo)
    h = hmac.new(secret_key, cleartext + iv, hash_fn)

    return {
        'ciphertext_64': ciphertext_64,
        'aes_iv_64': base64.b64encode(iv),
        'kdf_salt_64': base64.b64encode(salt),
        'kdf_algo': 'PBKDF2', # this is currently the only choice
        'kdf_iter': kdf_iterations,
        'hmac_hex': h.hexdigest(),
        'hmac_algo': hmac_algo,
    }


def sym_decrypt_cfb_128(secret_key, ciphertext_64, aes_iv_64,
                        kdf_salt_64, hmac_hex, hmac_algo, kdf_iterations):
    """
    Decrypt an AES-128-CFB ciphertext encypted with sym_decryt_cfb().
    """
    secret_key = force_bytes(secret_key)
    assert hmac_algo in getattr(
        hashlib, 'algorithms_available',
        getattr(hashlib, 'algorithms', [])
    )
    iv = base64.b64decode(aes_iv_64)
    salt = base64.b64decode(kdf_salt_64)
    ciphertext = base64.b64decode(ciphertext_64)

    # derive the original AES 16-byte key in exactly the same way as before
    key = pbkdf2_derive_key('sha1', secret_key, salt, kdf_iterations, 16)
    crypto = AES.new(key, mode=AES.MODE_CFB, IV=iv)

    # decrypt
    cleartext = crypto.decrypt(ciphertext)

    # check that the data has not been tampered with / the password is correct
    hash_fn = getattr(hashlib, hmac_algo)
    mac = hmac.new(secret_key, cleartext + iv, hash_fn)
    if not constant_time_compare(force_bytes(mac.hexdigest()), force_bytes(hmac_hex)):
        raise ValueError("Password incorrect or data corrupt")
    return cleartext


def sym_decrypt_cfb_dict(secret_key, crypto_dict):
    return sym_decrypt_cfb_128(
        secret_key,
        crypto_dict['ciphertext_64'],
        crypto_dict['aes_iv_64'],
        crypto_dict['kdf_salt_64'],
        crypto_dict['hmac_hex'],
        crypto_dict['hmac_algo'],
        crypto_dict['kdf_iter']
    )



from __future__ import absolute_import, unicode_literals

import mock

from django.test import TestCase

from .models import (
    TestVidyoRealRPC,
    create_test_admin,
)


class RPCWrapperFunctionalTests(TestCase):
    def test_rpc_oydiv_rpc_instantiate_rpc(self):
        with mock.patch('oydiv_rpc.user.User', autospec=True) as user:
            v = TestVidyoRealRPC(_admin_model=create_test_admin())
            v.password = 'asdfsdfsadf'
            v.portal_displayname = 'wingus'
            v.portal_name = 'dingus'
            v.portal_extension = '234234'

            # convert to string to force evaluation of SimpleLazyObject
            str(v.admin_rpc.rpc)

            self.assertEqual(user.call_count, 1)
            self.assertIn(user.call_args[0][0], 'myhost.example.com')

    def test_rpc_create(self):
        with mock.patch('oydiv_rpc.user.User.create', autospec=True), \
            mock.patch('oydiv_rpc.user.User', autospec=True) as user:
            v = TestVidyoRealRPC(_admin_model=create_test_admin())
            v.password = 'asdfsdfsadf'
            v.portal_displayname = 'wingus'
            v.portal_name = 'dingus'
            v.portal_extension = '23241'

            user.return_value.create.return_value = 1337

            v.save()

            self.assertEqual(user.method_calls.count(mock.call().create()), 1)
            self.assertEqual(v.vidyo_entity_id, 1337)

    def test_rpc_delete(self):
        with mock.patch('oydiv_rpc.user.User', autospec=True) as user:
            v = TestVidyoRealRPC(_admin_model=create_test_admin())
            v.password = 'sadfjljasldgds'
            v.portal_displayname = 'wingus'
            v.name = 'dingus'
            v.portal_extension = '234234'
            v.save()
            v.delete()
            self.assertEqual(user.method_calls.count(mock.call().delete()), 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dj_oydiv', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vidyoadmin',
            name='user_location_tag',
            field=models.CharField(default=b'Node4', help_text='Default location tag for new users (router affinity) formultiple router configuration.', max_length=128, verbose_name='Location Tag'),
        ),
        migrations.AlterField(
            model_name='vidyoadmin',
            name='user_proxy',
            field=models.CharField(default=b'ExternalProxy VR2', help_text='Default Proxy name for TCP tunneling of conferences', max_length=128, verbose_name='Default Proxy'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import


def get_client_ip(request):
    """Get the IP address of the client, taking into account the X-Forwarded-For header"""
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        return x_forwarded_for.split(',')[0]
    return request.META.get('REMOTE_ADDR')



from __future__ import absolute_import, unicode_literals

import functools
import logging

from django.core.exceptions import PermissionDenied

logger = logging.getLogger(__name__)


def _printfn(fn, alternative, *args):
    warning = '%s %s is deprecated. Use %s instead' % (
        str(fn.__class__) + '.', str(fn.__name__) + '()', alternative
    )
    s = ''.join([str(arg) + ' ' for arg in args])
    logger.warn('[DEPRECATION WARNING]:%s, %s', warning, s)


def deprecated_method(method, alternative=None, warnfn=_printfn):
    @functools.wraps(method)
    def _wrapped(obj=None, *args, **kwargs):
        warnfn(method, alternative)
        if obj:
            return method(obj, *args, **kwargs)
        return method(*args, **kwargs)
    return _wrapped


def permission_denied_wrapper(method):
    @functools.wraps(method)
    def _wrapped(*args, **kwargs):
        logger.warn("permission to access %r(*%r, **r)" % (
            method, args, kwargs)
        )
        raise PermissionDenied()
    return _wrapped



from __future__ import absolute_import, unicode_literals

from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.test.client import RequestFactory
from django.test import TestCase
from django.contrib.admin import site
from django.apps.registry import apps

from .models import TestVidyo1, TestVidyoMock

from dj_oydiv.models import VidyoAdmin
from dj_oydiv.admin.vidyoadmin import VidyoAdminForm


class _AdminFormTestsBase(TestCase):

    def setUp(self):
        self.user = User.objects.create_superuser(
            username='testuser',
            password='password',
            email='user@example.com'
        )
        rf = RequestFactory()

        self.base_request = rf.post(
            '/',
            {
                'dj_auth_user': str(self.user.pk),
                'portal_user': 'admin',
                'portal_host': 'example.com',
                'portal_prefix': 123,
                'user_proxy': 'proxy1',
                'user_group': 'Default',
                'user_location_tag': 'Default',
                'replay_host': 'example.com',
                'cdr_host': 'example.com',
                'cdr_password': 'This field is required.',
                'cdr_port': '3306',
                'cdr_user': 'admin',
                'portal_password': 'password',
                'confirm_portal_password': 'password',
                'current_admin_password': 'password',
            },
            user=self.user
        )

        self.base_request.user = self.user

    def tearDown(self):
        self.user.delete()

    def test_good_form_validates(self):
        form = VidyoAdminForm(self.base_request.POST)
        self.assertTrue(form.is_valid())


class PasswordChangeFormTests(_AdminFormTestsBase):
    """
    Test the admin form password changes and model setup, etc.
    see ``dj_oydiv.admin.monkeypatch_password_form()`` for overrides.
    """

    def test_base_request_form_valid(self):
        """ensure that we have a base level of sanity"""
        form = VidyoAdminForm(self.base_request.POST)
        self.assertTrue(form.is_valid())

    def test_password_change_form_contrib_auth_update(self):
        """
        We're overriding the standard ``save`` method on the
        ``django.contrib.admin.forms.PasswordChangeForm``, but we want to make sure
        that it is still being called, and properly updating the database.
        """
        pass

    def test_password_change_form_no_commit_ignored(self):
        """
        If ``PasswordChangeForm.save(commit=False)``, then we need to not update all
        the vidyoadmin objects.
        """
        pass

    def test_password_change_form_save_fails_rollback(self):
        """
        If  ``PasswordChangeForm.save()`` fails for any reason, we need to have a safe
        rollback strategy.
        """
        pass


class TestAdminFormValidators(_AdminFormTestsBase):

    def test_base_input_valid(self):
        form = VidyoAdminForm(self.base_request.POST)
        self.assertTrue(form.is_valid())
        form.save()

    def test_ip_input_invalid_ipv4(self):
        self.base_request.POST['portal_host'] = '999.999.999.999'
        form = VidyoAdminForm(self.base_request.POST)
        self.assertFalse(form.is_valid())

    def test_ip_input_valid_ipv4(self):
        self.base_request.POST['portal_host'] = '192.168.1.100'
        form = VidyoAdminForm(self.base_request.POST)
        self.assertTrue(form.is_valid())

    def test_ip_input_invalid_ipv6(self):
        self.base_request.POST['portal_host'] = 'xxxx::'
        form = VidyoAdminForm(self.base_request.POST)
        self.assertFalse(form.is_valid())

    def test_hostname_input_accept_hostname_only(self):
        """
        People have a tendency to conflate ``hostname`` with a full uri.
        Here we assert that anything but a hostname causes validation failure.
        """
        self.base_request.POST['portal_host'] = 'https://example.com'
        form = VidyoAdminForm(self.base_request.POST)
        self.assertFalse(form.is_valid())

    def test_domain_invalid_domain(self):
        self.base_request.POST['portal_host'] = 128 * 'a'
        form = VidyoAdminForm(self.base_request.POST)
        self.assertFalse(form.is_valid())

    def test_save_request_without_password_validation_error(self):
        del self.base_request.POST['portal_password']
        with self.assertRaises(ValueError):
            form = VidyoAdminForm(self.base_request.POST)
            form.save()

    def test_request_without_password_is_invalid(self):
        del self.base_request.POST['portal_password']
        form = VidyoAdminForm(self.base_request.POST)
        self.assertFalse(form.is_valid())


class VidyoUserGenericFinderTests(TestCase):
    """
    The admin dicoverer is supposed to find all instances of VidyoUserBase and register them
    with the django admin.
    """
    def test_admin_registry(self):
        self.assertIn(VidyoAdmin, site._registry)

    def test_vidyouserbase_registry(self):
        self.assertIn(TestVidyoMock, apps.get_models())
        self.assertIn(TestVidyo1, apps.get_models())



from __future__ import absolute_import, unicode_literals

import string

from django.utils.crypto import get_random_string

from ..config import config

_DEFAULT_RAND_LENGTH = config.DEFAULT_PASSWORD_LENGTH


def random_string(length=_DEFAULT_RAND_LENGTH, num=True, special=True, letters=None):
    alphabet = string.ascii_letters
    if(num):
        alphabet += string.digits
    if(special):
        alphabet += '!"$%^&*()_+=-[]{}#@~,./<>?'
    return get_random_string(length, letters or alphabet)


def random_ascii_letters(length=_DEFAULT_RAND_LENGTH):
    return random_string(length, letters=string.ascii_letters)


def random_digits(length=_DEFAULT_RAND_LENGTH):
    return random_string(length, letters=string.digits)



from setuptools import setup, find_packages
import itertools
import os
import sys

PKGNAME = 'dj_oydiv'
DIR = os.path.dirname(__file__)


def version():
    """Single source version information without importing the main project"""
    l = {}
    g = {}
    with open(os.path.join(DIR, PKGNAME, 'version.py'), 'rb') as f:
        code = compile(f.read(), 'version.py', 'exec')
        exec(code, g, l)

    return l['VERSION']


with open(os.path.join(DIR, 'README.rst'), 'r') as readme:
    long_description = readme.read()


install_requires = ['django<1.9','pyCrypto', 'arrow', 'oydiv_rpc']
if sys.version_info < (3, 4):
    install_requires.append('enum34')


setup(
    packages=find_packages(exclude=['tests']),
    name=PKGNAME,
    version=version(),
    licence='MIT',
    author_email='dev@ajenta.net',
    url='https://github.com/ajenta/dj-oydiv',
    author='ajenta',
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'wsauth': ['spyne', 'lxml'],
    },
    tests_require=['tox', 'coverage', 'flake8']
)



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib.auth.forms import PasswordChangeForm
from django.core.exceptions import ObjectDoesNotExist

logger = logging.getLogger(__name__)


def monkeypatch_password_form():
    """
    Don't look at me. I'M HIDEOUS!
    On import this module monkey-patches the password change form used within
    the django admin interface, so that when a user changes her password, any
    vidyoadmin objects, and associated VidyoUserBase keys are updated.
    """
    logger.debug("Monkeypatching django admin password change form")
    oldsave = PasswordChangeForm.save

    def newsave(self, commit=True):
        logger.debug("django admin password change form: attempting to update VidyoAdmin passwords")
        # assume we don't have to do any checking whether the passwords match.
        old_key = self.cleaned_data["old_password"]
        new_key = self.cleaned_data["new_password1"]
        try:
            self.user.vidyoadmin.decrypt(old_key)
            logger.debug("successfully decrypted old VidyoAdmin object")
            self.user.vidyoadmin.change_key(old_key, new_key)
        except (ObjectDoesNotExist, AttributeError):
            # If the user does not have a vidyoadmin, we don't care
            pass
        return oldsave(self, commit)

    PasswordChangeForm.save = newsave



import logging

from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.debug import sensitive_variables
from django.core.exceptions import PermissionDenied
from django.utils.crypto import constant_time_compare
from django.utils.lru_cache import lru_cache
from django.utils.encoding import force_text
from django.contrib.auth import authenticate

from spyne.decorator import srpc
from spyne.service import ServiceBase
from spyne.model.primitive import Boolean, Unicode
from spyne.protocol.soap import Soap11
from spyne.server.django import DjangoView
from spyne.application import Application

from ..auth.basic import http_basic_credentials, HttpResponseBasicChallenge
from ..utils.request import get_client_ip
from ..config import config

logger = logging.getLogger(__name__)


__all__ = [
    "portal_wsauth_as_view", "settings_portal_authenticator",
    "deny_all"
]


# The class generation here is expensive, so cache its return value.
@lru_cache()
def portal_wsauth_as_view(user_authenticator, portal_authenticator):
    """
    Create a view that validates a user login request from a portal.
    ``portal_authenticator`` and ```user_authenticator`` are both callbacks
    to functions taking two positional arguments, (``username``, ``password``)
    ```user_authenticator`` authenticates a user against the Portal.
    ``portal_authenticator`` validates whether the VidyoPortal itself is allowed
    to request validation for a particular user (essentially acting like an API key).

    Authentication callbacks should avoid setting cookies, or actually authenticating
    a user to the system, but simply check the user/password combination is valid
    to avoid leaving uneccessary sessions lying around.

    e.g.
    >>> from django.contrib.auth.models import User

    >>> def my_callback(username, password):
    ...     try:
    ...         user = User.objects.get(username=username)
    ...         if not user.is_active or not user.is_staff:
    ...             return False
    ...         return user.check_password(password)
    ...     except User.DoesNotExist:
    ...         return False

    """
    @sensitive_variables('username', 'password')
    def authenticator_wrapper(callback):
        """
        Close over the original callback with a simple exception handling wrapper
        to protect against inadvertent information leakage in case the supplied
        callback gets its knickers in a twist, and raises something like ``DoesNotExist``
        which will subsequently be marshalled into a 500 rather than returning False.
        That situation will allow for very easy object fingerprinting from remote.
        We don't want it.
        """
        def inner(username, password):
            try:
                return callback(username, password)
            # Catching bare ``Exception`` here
            except Exception as e: # NOQA
                logger.exception("user callback failed with exception:%s", e)
                return False

        return inner

    user_authenticator = authenticator_wrapper(user_authenticator)
    portal_authenticator = authenticator_wrapper(portal_authenticator)

    class VidyoPortalAuthSoapView(DjangoView):
        """
        Checks the user/password header sent as part of the HTTP basic auth
        request against the user's VidyoPortal validator, before dispatching
        to the Spyne SOAP handler.
        """
        @sensitive_variables('username', 'password')
        def dispatch(self, request, *args, **kwargs):
            # Vidyo sends the credentials in the HTTP_AUTHORIZATION header
            if request.META['REQUEST_METHOD'] == 'POST':
                try:
                    username, password = map(
                        force_text, http_basic_credentials(request)
                    )
                except (AttributeError, ValueError):
                    return HttpResponseBasicChallenge()
                if not portal_authenticator(username, password):
                    logger.info(
                        "failed authentication for '%s' from %s ",
                        username,
                        get_client_ip(request)
                    )
                    return HttpResponseBasicChallenge()

            return super(VidyoPortalAuthSoapView, self).dispatch(request, *args, **kwargs)

    return VidyoPortalAuthSoapView.as_view(
        application=Application([
            type(
                'AuthenticationService',
                (ServiceBase,),
                {
                    'AuthenticationRequest': srpc(
                        Unicode,
                        Unicode,
                        _returns=Boolean,
                        _out_message_name='AuthenticationResponse',
                        _out_variable_name='passed'
                    )(user_authenticator)
                }
            )],
            tns='http://ws.vidyo.com/authentication',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )
    )


@sensitive_variables()
def settings_portal_authenticator(username, password):
    """
    Check an incoming portal webservices authentication request against
    a user and password from the project's settings.py
    ``settings.OYDIV_WSAUTH_CREDENTIALS``, should be a dictionary-like object of
    password values indexed by username.
    """
    try:
        return constant_time_compare(config.WSAUTH_PORTAL_CREDENTIALS[username], password)
    except KeyError:
        logger.info("rejected portal auth request for %r", username)
        return False


_deny = lambda u, p: False


deny_all = csrf_exempt(
    portal_wsauth_as_view(_deny, settings_portal_authenticator)
)
deny_all.__doc__ = (
    """
    A default deny policy for all incoming authentication requests.
    The Portal itself is verified using ``settings_portal_authenticator``.
    This is usefaul if you're handling all endpoint authentication yourself with
    the webservices API, and will provide users an extra barrier to accessing
    their account configuration on the portal.
    """
)


def _django_auth_validator(staff_only):
    def inner(username, password):
        try:
            user = authenticate(username=username, password=password)
        except PermissionDenied:
            return False
        if not user:
            return False
        if user.is_anonymous():
            return False
        if staff_only and not user.is_staff:
            return False
        if user.is_active:
            return True
        return False

    return inner


django_auth_staffonly_view = csrf_exempt(
    portal_wsauth_as_view(_django_auth_validator(staff_only=True), settings_portal_authenticator)
)
django_auth_staffonly_view.__doc__ = (
    """
    Try and authenticate a user with ``django.contrib.auth``.
    If the user is not a staff member, the portal authentication
    will be denied.

    Portal SOAP calls are validated with ``settings_portal_authenticator``
    """
)

django_auth_user_view = csrf_exempt(
    portal_wsauth_as_view(_django_auth_validator(staff_only=False), settings_portal_authenticator)
)
django_auth_user_view.__doc__ = (
    """
    Try and authenticate a user with the `django.contrib.auth`.
    Portal SOAP calls are validated with ``settings_portal_authenticator``.
    """
)



from __future__ import absolute_import, unicode_literals

import logging

from django.core.exceptions import ValidationError, PermissionDenied
from django import forms
from django.db import router
from django.contrib import messages
from django.contrib.admin.actions import delete_selected
from django.utils.translation import ugettext as _
from django.utils.encoding import force_text
from django.contrib import admin
try:
    # django >= 1.8
    from django.contrib.admin.utils import get_deleted_objects
except ImportError:
    from django.contrib.admin.util import get_deleted_objects

from ..models import VidyoUserBase, VidyoAdmin
from ..utils.random import random_string

logger = logging.getLogger(__name__)


class _VidyoUserModelFormTemplate(forms.ModelForm):
    search_fields = ('client_id', 'name', 'displayName')
    user_password = forms.CharField(
        widget=forms.PasswordInput(),
        required=True,
        help_text=_("Enter password to confirm")
    )

    def clean(self, *args, **kwargs):
        """
        Make sure that the user is providing their password
        so we can encrypt/decrypt the model appropriately.
        """
        data = super(_VidyoUserModelFormTemplate, self).clean(*args, **kwargs)
        user_password = data.get('user_password')

        user = data.get('dj_auth_user', None) or self.current_user
        data['dj_auth_user'] = user
        #TODO add user permissions checking.
        #If user can create  models then they can assign to the
        #VidyoAdmin of their choice.
        #If the user can't create VidyoAdmin  objects, then
        if user_password:
            if not user.check_password(user_password):
                raise ValidationError(
                    {'user_password': [_("Admin Password is incorrect")]}
                )
        else:
            raise ValidationError(
                _("Password required")
            )
        return self.cleaned_data


class _VidyoUserModelAdminTemplate(admin.ModelAdmin):
    actions = ['delete_selected_vidyo_models']
    search_fields = ('portal_displayname', 'extension', 'client_id')
    list_display = (
        'portal_displayname', 'portal_extension', 'portal_name', 'client_id',
        'can_call', 'can_host', 'can_record', 'vidyo_entity_id'
    )

    def __init__(self, *args, **kwargs):
        """__init__ is overidden so that self is accessible to retrieve appname
        This is to avoid having to hardcode this in case it resides in a
        different namespace per project.
        """
        super(_VidyoUserModelAdminTemplate, self).__init__(*args, **kwargs)
        app_label = VidyoAdmin()._meta.app_label

        confirm = '%s/delete_selected_confirmation.html' % app_label
        delete = '%s/delete_confirmation.html' % app_label
        self.delete_selected_confirmation_template = confirm
        self.delete_confirmation_template = delete

    def get_actions(self, request):
        """we need to remove the deletion option to use our own instead of the default_proxy
        because the default does not require passwords"""
        actions = super(_VidyoUserModelAdminTemplate, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def delete_selected_vidyo_models(self, request, queryset):
        """The first half of this method is copied practically verbatim from
        django.contrib.admin.actions.delete_selected.  However, the actual deletion
        is done within this method because django.contrib.admin uses queryset.delete()
        for efficiency. That method precludes the use of RPC.
        """

        opts = self.model._meta

        # Check that the user has delete permission for the actual model
        if not self.has_delete_permission(request):
            raise PermissionDenied()

        using = router.db_for_write(self.model)

        # Populate deletable_objects, a data structure of all related objects that
        # will also be deleted.
        deletable_objects, perms_needed, protected = get_deleted_objects(
            queryset, opts, request.user, self.admin_site, using
        )
        # If the POST querydict has get('post'), that means the submit button
        # has been submitted: Go ahead and delete
        if request.POST.get('post'):
            password = request.POST.get('password')
            if not password:
                self.message_user(request, _("Password Required"), messages.ERROR)
                return None
            if perms_needed:
                raise PermissionDenied()

            deleted_models = 0
            for obj in queryset:
                obj_display = force_text(obj)
                self.log_deletion(request, obj, obj_display)
                try:
                    obj.decrypt(password)
                except ValueError:
                    continue
                obj.delete()
                deleted_models += 1

            if len(queryset) > deleted_models:
                self.message_user(
                    request,
                    _("couldn't delete all %ss: some were skipped") % str(type(self)),
                    messages.WARNING
                )
            else:
                self.message_user(
                    request,
                    _("Successfully deleted %(number)s %(type)ss") %
                    {
                        'number': deleted_models,
                        'type': self.model
                    },
                    messages.SUCCESS
                )
            return None
        # If the POST doesn't contain 'post' key that means we're not yet ready
        # to do the full deletion, so let django.contrib.admin render the
        # confirmation page with our overridden template
        else:
            return delete_selected(self, request, queryset)

    def delete_model(self, request, object_id, extra_context=None):
        # if request.POST is set, the user already confirmed deletion
        if not self.has_delete_permission(request):
            raise PermissionDenied()

        password = request.POST.get('password')
        if not password:
            self.message_user(request, _("Password Required"), messages.ERROR)
            return None
        try:
            object_id.decrypt(password)
        except ValueError:
            self.message_user(request, _("Wrong Password"), messages.ERROR)
            return None
        try:
            object_id.delete()
        except Exception:
            self.message_user(
                request,
                _("An unknown error occurred during deletion"),
                messages.ERROR
            )
            return None
        self.message_user(request, _("Successfully deleted"), messages.SUCCESS)
        return None

    def get_form(self, request, obj=None, **kwargs):
        form = super(_VidyoUserModelAdminTemplate, self).get_form(request, obj, **kwargs)
        form.current_user = request.user
        return form

    def save_model(self, request, obj, form, change):
        if change:
            if not self.has_change_permission(request):
                logger.exception("Permissions error %s" % request)
                raise PermissionDenied()
            logger.exception("Cannot update VidyoUserBase instance")
            return self.message_user(
                request,
                _("Updating VidyoUser instances is not yet supported"),
                messages.ERROR
            )
        else:
            if not self.has_add_permission(request):
                logger.exception("Permissions error %s" % request)
                raise PermissionDenied()

        if not form.cleaned_data.get('dj_auth_user'):
            logger.debug(
                "dj_auth_user not selected in admin form:"
                "falling back to logged in user<%s>:%s", request.user, obj
            )

        password = form.cleaned_data.get('user_password')
        obj._admin_model = form.cleaned_data.get('dj_auth_user').vidyoadmin
        obj._admin_model.decrypt(password)
        if change:
            pass
            # FIXME
#            obj.change_password(password, random_string())
        else:
            obj.set_password(password, random_string())
#        obj.clean()
        obj.save()


class _VidyoUserGenericModelFormMeta(type):
    def __new__(cls, clsname, bases, attrs):
        if len(bases) != 1:
            raise ValueError(
                _("VidyoUserAdminForm requires exactly one VidyoUserBase subclass")
            )
        assert issubclass(bases[0], VidyoUserBase)

        # we cast the name with ``str()`` because Python2.x requires bytestrings for typenames
        # Python3 is happy with a unicode literal, and str() is a no-op in py3.
        class_meta = type(
            str('VidyoAdminModelFormMeta'),
            (object,),
            {'model': bases[0], 'exclude': ()}
        )
        class_dict = dict({'Meta': class_meta})

        #add user overrides (if given)
        class_dict.update(attrs)
        model_form = type(
            str(bases[0].__name__ + 'ModelForm'),
            (_VidyoUserModelFormTemplate,),
            class_dict
        )
        return model_form


class VidyoUserGenericModelAdminMeta(type):
    def __new__(cls, clsname, bases, attrs):
        if len(bases) != 1:
            raise ValueError(
                _("VidyoUserAdminForm requires exactly one base class")
            )

        class_meta = type(str('VidyoAdminModelAdminMeta'), (object,), {'model': bases[0]})
        class_dict = dict({'Meta': class_meta})
        class_dict['form'] = _VidyoUserGenericModelFormMeta(clsname, bases, attrs)
        class_dict.update(attrs)
        model_admin = type(
            str(bases[0].__name__ + 'ModelAdmin'),
            (_VidyoUserModelAdminTemplate,),
            class_dict
        )
        return model_admin


def register_all_vidyo_models(**attr_dict):
    classes = VidyoUserBase.all_vidyo_user_classes()
    model_admins = [
        VidyoUserGenericModelAdminMeta(x.__name__, (x,), attr_dict)
        for x in classes
    ]

    for klass, model_admin in zip(classes, model_admins):
        logger.debug("registering %r:%r with the django admin", klass, model_admin)
        admin.site.register(klass, model_admin)



from __future__ import absolute_import, unicode_literals

import mock

from django.test import TestCase, SimpleTestCase

from . import PACKAGE_BASE, VIDYO_PORTAL_PASSWORD
from .models import (
    TestVidyoMock,
    TestVidyo1,
    create_test_admin,
    create_multiple_children,
)

from dj_oydiv.models import VidyoUserBase, VidyoAdmin
from dj_oydiv.utils.random import random_ascii_letters


"""
Factory function tests.
The factory functions defined in the factory module are expected to return a
``django.model.Model`` or VUser/VReplay/VCDRInfo object depending on the function.
Most take a client ID and a class type, and retrieve the appropriate model.

Multiple objects should *never* be returned.
"""


class AdminWithMultipleChildrenMixin(object):
    N_CHILD_USERS = 8

    def setUp(self):
        self.admin = create_test_admin()

        # now create and save a bunch of derivatives of VidyoUserBase
        self.test0_users = [(x + 1) * 'b' for x in range(self.N_CHILD_USERS)]
        create_multiple_children(self.admin, self.test0_users, TestVidyoMock, self.N_CHILD_USERS)

        self.test1_users = [(x + 1) * 'a' for x in range(self.N_CHILD_USERS)]
        create_multiple_children(self.admin, self.test1_users, TestVidyo1, self.N_CHILD_USERS)

    def tearDown(self):
        TestVidyoMock.objects.all().delete()


class VidyoAdminMethodTests(AdminWithMultipleChildrenMixin, TestCase):
    def test_password_change_propagates_children(self):
        self.admin.change_key(VIDYO_PORTAL_PASSWORD, 'mynewpassword')
        self.admin.save()
        for x in list(TestVidyoMock.objects.all()) + list(TestVidyo1.objects.all()):
            with self.assertRaises(ValueError):
                x.decrypt('mypassword')

            x.decrypt('mynewpassword')
            self.assertEqual(x.password, 'mysecretdata')

    def test_all_vidyo_user_classes_complete(self):
        for x in [TestVidyoMock, TestVidyo1]:
            if x not in VidyoUserBase.all_vidyo_user_classes():
                raise Exception("Non-abstract models not registered")

    def test_all_vidyo_users_complete(self):
        """make sure the expected number of TestVidyoMock/TestVidyo1 children are listed"""
        self.assertEqual(self.N_CHILD_USERS * 2, len(list(self.admin.all_vidyo_users())))

    def test_all_vidyo_users_match_admin(self):
        """
        Make sure that the ``a.all_vidyo_users``method does not return any objects in
        ``b.all_vidyo_users()``
        """
        a = create_test_admin(admin='donny')
        # We must compare lists, rather than iterators
        self.assertNotEqual(
            sorted(list(a.all_vidyo_users()), key=lambda x: x.pk),
            sorted(list(self.admin.all_vidyo_users()), key=lambda x: x.pk)
        )
        self.assertEqual(len(list(a.all_vidyo_users())), 0)

    def test_all_vidyo_users_objects(self):
        for x in self.test0_users:
            u = TestVidyoMock.objects.get(client_id=x)
            self.assertIn(u, self.admin.all_vidyo_users())
        for x in self.test1_users:
            u = TestVidyo1.objects.get(client_id=x)
            self.assertIn(u, self.admin.all_vidyo_users())

    def test_ssl_propagation(self):
        """
        assert that the value returned by admin.use_ssl() is equal to admin.ssl
        among all related VidyoAdmin/VidyoUserBase objects.
        """
        self.admin.ssl = True
        self.admin.save()
        self.assertTrue(self.admin.use_ssl())
        for x in self.admin.all_vidyo_users():
            self.assertTrue(x.use_ssl())
        self.admin.ssl = False
        self.admin.save()
        self.assertFalse(self.admin.use_ssl())
        for x in self.admin.all_vidyo_users():
            self.assertFalse(x.use_ssl())

    def test_user_count(self):
        self.assertEqual(self.N_CHILD_USERS * 2, self.admin.user_count)

    def test_user_count_multiple_admins(self):
        a = create_test_admin(admin='donny')
        self.assertFalse(a.user_count, 0)
        self.assertEqual(self.N_CHILD_USERS * 2, self.admin.user_count)


class ModelDefaultsTests(SimpleTestCase):
    def test_bare_save_new_password(self):
        """
        just calling save should be enough to generate a random password
        on a user model
        """
        with mock.patch(
            PACKAGE_BASE + '.utils.random.get_random_string',
            mock.Mock(return_value='thisisrandom')
        ):
            ad = VidyoAdmin()
            ad.secret_key = 'pizzatime'
            ad.password = 'whatever'
            ad.save()

            model = TestVidyoMock(_admin_model=ad)
            model.secret_key = 'pizzatime'
            model.save()
            self.assertEqual(model.password, 'thisisrandom')

    def test_new_model_pin_not_equal(self):
        alice = TestVidyoMock(_admin_model=VidyoAdmin())
        bob = TestVidyoMock(_admin_model=VidyoAdmin())
        self.assertNotEqual(bob.pin, alice.pin)



from __future__ import absolute_import, unicode_literals

import uuid
import mock
import random

from django.db import models

from dj_oydiv.models import (
    VidyoUserBase,
    VidyoAdmin,
    CryptoText
)
from . import VIDYO_PORTAL_PASSWORD, PACKAGE_BASE


def side_effect(*args, **kwargs):
    return random.randint(1, 1000000)


class CryptoTextTest(CryptoText):
    pass


class TestVidyoMock(VidyoUserBase):
    secret_key = VIDYO_PORTAL_PASSWORD
    client_id = models.CharField(max_length=64)

    def save(self, *args, **kwargs):
        with mock.patch(PACKAGE_BASE + '.rpc_wrapper.user') as mocked:
            mocked.create.side_effect = side_effect
            super(TestVidyoMock, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        with mock.patch(PACKAGE_BASE + '.rpc_wrapper.user'):
            super(TestVidyoMock, self).save(*args, **kwargs)

    @staticmethod
    def normalize_client_id(client_id):
        return client_id

    def get_name(self):
        return str(uuid.uuid4())

    def get_extn(self):
        return random.randint(1, 10000000)


class TestVidyo1(VidyoUserBase):
    secret_key = VIDYO_PORTAL_PASSWORD
    client_id = models.CharField(max_length=64)

    def save(self, *args, **kwargs):
        with mock.patch(PACKAGE_BASE + '.rpc_wrapper.user') as mocked:
            mocked.create.side_effect = lambda *_, **__: random.randint(1, 1000000)
            super(TestVidyo1, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        with mock.patch(PACKAGE_BASE + '.rpc_wrapper.user'):
            super(TestVidyo1, self).save(*args, **kwargs)

    @staticmethod
    def normalize_client_id(client_id):
        return client_id

    def get_name(self):
        return str(uuid.uuid4())

    def get_extn(self):
        return random.randint(1, 10000000)


class TestVidyoRealRPC(VidyoUserBase):
    secret_key = VIDYO_PORTAL_PASSWORD
    client_id = models.CharField(max_length=64)

    @staticmethod
    def normalize_client_id(client_id):
        return client_id

    def get_name(self):
        return str(uuid.uuid4())

    def get_extn(self):
        return random.randint(1, 10000000)


def save_then_retrieve(model, **kwargs):
    T = type(model)
    model.save()

    pk = model.id

    t = T.objects.get(pk=pk)
    for k, v in kwargs.items():
        setattr(t, k, v)
    return t


def create_test_admin(ssl=False, dj_auth_user=None, password=VIDYO_PORTAL_PASSWORD, admin='admin'):
    ad = VidyoAdmin(
        portal_user=admin,
        portal_host='myhost.example.com',
        replay_host='myhost.example.com',
        portal_prefix='431',
        ssl=ssl
    )
    if dj_auth_user:
        ad.dj_auth_user = dj_auth_user
    ad.set_password(password, VIDYO_PORTAL_PASSWORD)
    ad.save()
    return ad


def create_multiple_children(admin, client_ids, child_class=TestVidyoMock, count=10):
    for x in range(count):
        u = child_class(_admin_model=admin)
        u.vidyo_entity_id = str(x + 1000)
        u.client_id = client_ids[x]
        u.extension = str(x)
        u.name = str(x)
        u.set_password(admin.secret_key, 'mysecretdata')
        u.save()


def create_only_child(admin, klass=TestVidyoMock, save=False):
    """creates a TestVidyoModel, with the expectation that no id collisions will happen"""
    u = klass(_admin_model=admin)
    u.vidyo_entity_id = '1'
    u.client_id = '1'
    u.extension = admin.portal_prefix + '1'
    u.name = 'test'
    u.set_password(admin.secret_key, 'mysecretdata')
    if save:
        u.save()
    return u
